/*
bhbarcode.h
Copyright (C)2001 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.
*/

#ifndef _BHBARCODE_H_
#define _BHBARCODE_H_

#include <libxml/tree.h>
#include <libxml/parser.h>
#include <bhdev.h>

/**
 * @defgroup hwlib Hardware Libraries
 * @{
 */

namespace bhpos{ 

/**
 * @defgroup bhbarcode Barcode Scanner
 * @{
 *
 * \ingroup hwlib
 * @{	
 */
//!Barcode Device Type
enum BarcodeDeviceType{
   BCDEV_TYPE_NONE=0,
   BCDEV_TYPE_KEYBOARD,
   BCDEV_TYPE_SERIAL,
   BCDEV_TYPE_USB,
   BCDEV_TYPE_PORTABLE
};

//!Barcode Format
enum BarcodeFormat{
   BCF_EXACTLENGTH,  //! - fit to exact length
   BCF_ZEROPADDLEFT, //! - zero padd to the left to specified length (implies right justified barcode)
   BCF_ZEROPADDRIGHT //! - zero padd to the right to specified length (implies left justified barcode)
};

//!Barcode Type
enum BarcodeType{
	UNKNOWN     = 0, /*! - Unknown type */
	CODABLOCK,       /*! - codablock (unsupported)*/
	CODE1,			  /*! - code 1 (unsupported)*/
	CODE16K,         /*! - code 16k (unsupported)*/
	CODE11,          /*! - code 11 (unsupported)*/
	CODE39,          /*! - code 39 (unsupported)*/
	CODE2OF5,        /*! - code 2 of 5 (unsupported)*/
	CODE49,          /*! - code 49 (unsupported)*/
   CODE93,          /*! - code 93 (unsupported)*/
   CODE128,         /*! - code 128 (unsupported)*/
   CODABAR,         /*! - codabar (unsupported)*/
   EAN8,            /*! - EAN-8 european article numbering 7 digits and a check digit.*/
   EAN8_2,          /*! - EAN-8 + 2 european article numbering 7 digits and a check digit, 
                       with 2 additional digits tacked on the end.*/
   EAN8_5,          /*! - EAN-8 + 5 european article numbering 7 digits and a check digit, 
                       with 5 additional digits tacked on the end.*/
   EAN13,           /*! - EAN-13 european article numbering 12 digits and a check digit, 
                       This is the default format.*/
   EAN13_2,         /*! - EAN-13 + 2 european article numbering 12 digits and a check digit, 
                       with 2 additional digits tacked on the end.*/
   EAN13_5,         /*! - EAN-13 + 5 european article numbering 12 digits and a check digit, 
                       with 5 additional digits tacked on the end.*/
   EAN128,          /*! - EAN-128 european article numbering alphanumeric (not supported)*/
   FIM,             /*! - FIM (not supported)*/
   ISBN,            /*! - ISBN international standard book numbering*/
   ISBN_5,          /*! - ISBN international standard book numbering, extra 5 digits*/
   EANISBN,         /*! - EAN-ISBN european article numbering for ISBN numbers*/
   I2OF5,           /*! - Interleaved 2 of 5 (not supported) */
   JAN8,            /*! - Japanese Article numbering same as EAN-8 \sa EAN_8*/
   JAN8_2,          /*! - Japanese Article numbering same as EAN-8+2 \sa EAN8_2*/
   JAN8_5,          /*! - Japanese Article numbering same as EAN-8+5 \sa EAN8_5*/
   JAN13,           /*! - Japanese Article numbering same as EAN-13 \sa EAN13*/
   JAN13_2,         /*! - Japanese Article numbering same as EAN-13+2 \sa EAN13_2*/
   JAN13_5,         /*! - Japanese Article numbering same as EAN-13+5 \sa EAN13_5*/
   MSI,             /*! - Plessey (not supported) */
   NI2OF5,          /*! - NI 2 of 5 (not supported) */
   PHARMA,          /*! - Pharmacy code (not supported) */
   POSTNET,         /*! - Post code (not supported) */
   UPCA,            /*! - Universal Product Code A */
   UPCA_2,
   UPCA_5,
   UPCE,
   UPCE_2,
   UPCE_5,
   BH_SHORTCODE6=100/*! - BananaPos 6 digit short code (for quick keyboard entries) */
};

namespace helper{  

/*! \brief A class to help barcode formatting/validation.
 */
class CBarcodeHelper
{
protected:
	char *data;
	BarcodeType	type;

	int GetNumber(char c);
	int LunBun();
	
public:
	CBarcodeHelper();
	CBarcodeHelper(const char *target_barcode);
	~CBarcodeHelper();
	/*!
		Set the barcode
		\param target_barcode a const char pointer
		\return 0 on sucess, non-zero on fail
	*/
	int Set(const char *target_barcode);
	/*!
		Returns the number of digits in the barcode
		\return number of digits in barcode
	*/
	int Size();
	/*!
		Check the barcode's validity
		\return true if barcode valid, false on invalid
	*/
	bool IsValid();
	/*!
		Format a barcode to another type, or a user defined type
		\param target_type an unsigned int indicating the target type
		\sa EAN13
		\param flags formats the output according to:
		\sa BCF_EXACTLENGTH, BCF_ZEROPADDLEFT, BCF_ZEROPADDRIGHT
		\param target_size size of the formated barcode in digits
		\return 0 on success.
	*/
	int FormatTo(BarcodeType target_type, unsigned int flags, int target_size);
	/*!
		Get the barcode
		\return pointer to a const string.
	*/
	const char *Get();
};

/*! \brief A class for handling sentinels. */
class CSentinel
{
public:
   enum SentinelStatus{
      SENTINEL_STATUS_IDLE=0,
      SENTINEL_STATUS_START,
      SENTINEL_STATUS_INSIDE,
      SENTINEL_STATUS_END,
      SENTINEL_STATUS_DATAVALID,
      SENTINEL_STATUS_ERROR
   };
   
protected:
   bool  bDropSentinels;
   int   len;
   char *data;

public:
   CSentinel();
   CSentinel(const CSentinel&);
   ~CSentinel();
   void operator=(const CSentinel&);
   bool operator==(const CSentinel&);
   bool operator<(const CSentinel&);
   int Set(char *, char *);
   SentinelStatus GetStatus();
   SentinelStatus Parse(char *);
};

}   // Helper

/*! \brief A class for handling barcode scanners. */
class CBarcode : public CBhDevice
{
private:
	BarcodeDeviceType bctype;

protected:
   int               status;
   helper::CSentinel start_sentinel;
   helper::CSentinel end_sentinel;
   int               max_data;
	
public:
	CBarcode(BhDevType);
	~CBarcode();
   void SetBcType(BarcodeDeviceType t) { bctype=t; }
   BarcodeDeviceType GetBcType() { return bctype; }
	int OpenScanner(const char *, BhDevNotifyType);
	void Input(int len);
};

namespace parser{

/*! \brief Barcode config parser. */
class CBarcodeParser : public CDeviceParser
{
	xmlNsPtr	ns;
	
public:
	CBarcodeParser();
	~CBarcodeParser();
	int Open(char *, CBarcode&);
	int ParseScanner(xmlNodePtr, CBarcode&);
};

}
/* @} 
	@} */
}

#endif //_BARCODER_
